#!/bin/sh
# next line is a comment in tcl \
exec wish "$0" ${1+"$@"}

########################################################################
# splinetest.tcl - a test of spline rendering using the Tkspline extension
#
# John Ellson - ellson@lucent.com 
########################################################################

set auto_path "[list [file dirname [info script]]] $auto_path"
package require Tkspline

# create canvas and frame for toolbar
pack [canvas .c -bg white -height 350] \
    -side top -fill both -expand true
pack [frame .f] -side top -fill x

# create a large polygon for hiding objects behind in case
# the dash patch is not available
set havedash 0
set hide [.c create polygon 0 0 0 350 350 350 350 0 -fill white]
if {! [catch {.c itemconfigure $hide -state disabled}]} {
    # if -state disabled worked then we have the dash patch and
    # we don't need the hide.

# COMMENT OUT next two lines to test in the without-dash-patch-mode even though
# dash patch is available to support Tkspline DLL
# e.g. for testing before tk8.0.5

    .c delete $hide
    set havedash 1

}

if {$havedash} {

    proc nextpoint {c wx wy} {
        global id oldx oldy
    
        if {[info exists id]} {
            unset id
        } {
            set id [$c find withtag current]
            if {[string compare $id {}]} {
            # if $id != {}   
            # then must be a Point because everything else is disabled
                set oldx [$c canvasx $wx]
                set oldy [$c canvasy $wy]
            } {
            # else not a Point
                unset id
            }
        }
    }
    
    proc motion {c wx wy} {
        global id oldx oldy
        set x [$c canvasx $wx]
        set y [$c canvasy $wy]
        if [info exists id] {
            foreach {items idx} [$c gettags $id] {break}
            # update single coord in three variants of shape
            foreach itm [split $items .] {
                   $c dchars $itm $idx
                   $c insert $itm $idx [list $x $y]
            }
            # move control point
            $c move $id [expr $x - $oldx] [expr $y - $oldy]
        }
        set oldx $x
        set oldy $y
    }
    
    # itemwithcontrolpoints - create a "line" or "polygon" with
    #    straight + spline + smooth  line variants and draggable 
    #    control points.
    #
    # c = canvas handle
    # item = "line" | "polygon"
    # coords = x y x y ... list of control point locations
    #
    proc itemwithcontrolpoints {c item coords} {
        if {[string compare $item "line"]} {
        # if $item != "line" (i.e $item == "polygon")
            set opts {-fill {} -outline black}
        } {
        # else $item == "line"
            set opts {}
        }
        set lid [eval $c create $item $coords $opts \
            -state disabled -tag Line]
        set slid [eval $c create $item $coords $opts \
            -smooth true -state disabled -tag Smooth]
        set bzlid [eval $c create $item $coords $opts \
            -smooth spline -state disabled -tag Spline]
        set idx 0
        foreach {x y} $coords {
            set id [$c create oval -2 -2 2 2 \
                -fill yellow -outline black \
                -tags [list $lid.$slid.$bzlid $idx Point]]
            $c move $id $x $y 
            incr idx 2
        }
    }
    
    # show objects by type (from toolbar buttons)
    proc show {typ show} {
        if {$show} {
            if {[string compare $typ "Point"]} {
            # if $typ != "Point"
                .c itemconfigure $typ -state disabled
            } {
            # else $typ == "Point"
                .c itemconfigure $typ -state normal
            }
        } {
            .c itemconfigure $typ -state hidden
        }
    }

} {
# else no dash patch

    proc nextpoint {c wx wy} {
        global id oldx oldy
    
        if {[info exists id]} {
            unset id
        } {
            set id [$c find withtag current]
            if {[string compare $id {}] 
            && ! [string compare [$c type $id] "oval"]} {
            # if $id != {} && $type == "oval"
                set oldx [$c canvasx $wx]
                set oldy [$c canvasy $wy]
            } {
            # else not a Point
                unset id
            }
        }
    }
    
    proc motion {c wx wy} {
        global id oldx oldy
        set x [$c canvasx $wx]
        set y [$c canvasy $wy]
        if [info exists id] {
            foreach {items idx} [$c gettags $id] {break}
            # update single coord in three variants of shape
            set idy [expr $idx + 1]
            foreach itm [split $items .] {
                eval $c coords $itm \
                    [lreplace [$c coords $itm] $idx $idy $x $y]
            }
            # move control point
            $c move $id [expr $x - $oldx] [expr $y - $oldy]
        }
        set oldx $x
        set oldy $y
    }
    
    # itemwithcontrolpoints - create a "line" or "polygon" with
    #    straight + spline + smooth  line variants and draggable 
    #    control points.
    #
    # c = canvas handle
    # item = "line" | "polygon"
    # coords = x y x y ... list of control point locations
    #
    proc itemwithcontrolpoints {c item coords} {
        if {[string compare $item "line"]} {
        # if $item != "line" (i.e $item == "polygon")
            set opts {-fill {} -outline black}
        } {
        # else $item == "line"
            set opts {}
        }
        set lid [eval $c create $item $coords $opts \
            -tag Line]
        set slid [eval $c create $item $coords $opts \
            -smooth true -tag Smooth]
        set bzlid [eval $c create $item $coords $opts \
            -smooth spline -tag Spline]
        set idx 0
        foreach {x y} $coords {
            set id [$c create oval -2 -2 2 2 \
                -fill yellow -outline black \
                -tags [list $lid.$slid.$bzlid $idx Point]]
            $c move $id $x $y 
            incr idx 2
        }
    }
    
    # show objects by type (from toolbar buttons)
    proc show {typ show} {
        global hide
        # without dash patch we just hide behind a large opaque object
        if {$show} {
            .c raise $typ $hide
        } {
            .c lower $typ $hide
        }
    }
}
    
proc print {} {
    global tcl_platform
    if {[string compare $tcl_platform(platform) "windows"]} {
    # if platform != "windows" ... then probably unix
        set chan [open "| lpr" w]
        .c postscript -channel $chan
        close $chan
    } {
    # else platform == "windows"
        if {[catch {package require Printer}] 
        && [catch {package require printer}]} {
            # Can't do this--no printer package
            puts stderr "No \"Printer\" package found"
        } {
            # Ask the user to select a printer
            set hdc [ printer dialog select ]
            # Now send raw postscript output to the device
            printer send -hdc $hdc -postscript \
                -data [.c postscript]
        }
    }
}

# populate toolbar
foreach typ {Exit Print} {
    set cmd [string tolower $typ]
    pack [button .f.$cmd \
        -text $typ -width 6 -command $cmd \
        -padx 0 -pady 0 -width 6] \
            -side right
}

foreach typ {Line Smooth Spline Point} {
    pack [checkbutton .f.[string tolower $typ] \
        -text $typ -variable var$typ -width 6 \
        -selectcolor yellow -command "show $typ \$var$typ" \
        -indicatoron false] \
            -side left -fill both
    set var$typ 1
}

# mouse bindings on canvas objects
bind .c <1> "nextpoint .c %x %y"
bind .c <Motion> "motion .c %x %y"

# populate with demo objects
itemwithcontrolpoints .c line \
    {50 50 50 100 100 100 100 50 100 20 150 20 150 50}
itemwithcontrolpoints .c polygon \
    {50 200 100 150 150 200 150 250 100 300 50 250}

set help [.c create text 10 125 \
    -anchor w \
    -text "You can drag the points\nwith mouse button 1"]
after 5000 {.c delete $help}
